/**
 * ProjectLaogai
 *
 * Copyright 2018  <seil0@mosad.xyz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 *
 */

package org.mosad.seil0.projectlaogai.fragments

import android.graphics.Color
import android.os.Bundle
import androidx.fragment.app.Fragment
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.LinearLayout
import org.jetbrains.anko.doAsync
import org.jetbrains.anko.uiThread
import org.mosad.seil0.projectlaogai.MainActivity
import org.mosad.seil0.projectlaogai.R
import org.mosad.seil0.projectlaogai.hsoparser.DataTypes
import org.mosad.seil0.projectlaogai.hsoparser.NotRetardedCalendar
import org.mosad.seil0.projectlaogai.uicomponents.LessonCardView
import org.mosad.seil0.projectlaogai.uicomponents.MensaDayCardView
import java.text.SimpleDateFormat
import java.util.*

/**
 * The timetable controller class
 * contains all needed parts to display and the timetable detail screen
 */
class TimeTableFragment : Fragment() {

    private lateinit var linLayoutTTFragment: LinearLayout
    private var mainActivity = MainActivity()

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {

        val view: View = inflater.inflate(R.layout.fragment_time_table, container, false)

        linLayoutTTFragment = view.findViewById(R.id.linLayout_TTFragment)

        addCurrentWeek()

        return view
    }

    /**
     * add the remaining days of the current week to the timetable screen
     */
    private fun addCurrentWeek() {
        val dayIndex = NotRetardedCalendar().getDayOfWeekIndex()
        val formatter = SimpleDateFormat("E dd.MM", Locale.GERMANY) // TODO change to android call when min api is 24
        val calendar = Calendar.getInstance()

        doAsync {

            uiThread {

                // add current weeks days
                for(day in dayIndex..5) {
                    val cardViewTimeTableDay = MensaDayCardView(context!!, null)
                    cardViewTimeTableDay.setDayHeading(formatter.format(calendar.time))

                    // for each lessen of the day
                    for((i, lesson) in mainActivity.getTimeTableCurrentWeek()[day].withIndex()) {
                        val lessonCardView = LessonCardView(context!!, null)
                        lessonCardView.setBackgroundColor(Color.TRANSPARENT)

                        lessonCardView.getTxtViewLesson().text = resources.getString(R.string.string_new_line, lesson.lessonSubject)
                        lessonCardView.getTxtViewLesson().append(lesson.lessonTeacher + "\n")
                        lessonCardView.getTxtViewLesson().append(lesson.lessonRoom)
                        lessonCardView.getTxtViewTime().text = DataTypes().getTime()[i]

                        if(lessonCardView.getTxtViewLesson().text.length > 2)
                            cardViewTimeTableDay.getLinLayoutMensaDay().addView(lessonCardView)
                    }

                    calendar.add(Calendar.DATE,1)
                    linLayoutTTFragment.addView(cardViewTimeTableDay)
                }

                // add next weeks days, max number  = dayIndex, if timetable was loaded
                if (mainActivity.getTimeTableNextWeek().isNotEmpty()) {
                    calendar.add(Calendar.DATE,1) // before this we are at a sunday (no lecture on sundays!)

                    for(day in 0..(dayIndex - 1)) {

                        val cardViewTimeTableDay = MensaDayCardView(context!!, null)
                        cardViewTimeTableDay.setDayHeading(formatter.format(calendar.time))

                        // for each lessen of the day
                        for((i, lesson) in mainActivity.getTimeTableNextWeek()[day].withIndex()) {
                            val lessonCardView = LessonCardView(context!!, null)
                            lessonCardView.setBackgroundColor(Color.TRANSPARENT)

                            lessonCardView.getTxtViewLesson().text = resources.getString(R.string.string_new_line, lesson.lessonSubject)
                            lessonCardView.getTxtViewLesson().append(lesson.lessonTeacher + "\n")
                            lessonCardView.getTxtViewLesson().append(lesson.lessonRoom)
                            lessonCardView.getTxtViewTime().text = DataTypes().getTime()[i]

                            if(lessonCardView.getTxtViewLesson().text.length > 2)
                                cardViewTimeTableDay.getLinLayoutMensaDay().addView(lessonCardView)
                        }

                        calendar.add(Calendar.DATE,1)
                        linLayoutTTFragment.addView(cardViewTimeTableDay)
                    }
                }

                // TODO if there is no lesson at one day , show a no lesson card
            }

        }
    }

    fun setMainActivity(mainActivity: MainActivity) {
        this.mainActivity = mainActivity
    }

}
